using System;
using System.Runtime.InteropServices; // For APIs
using System.Windows.Forms;

namespace SimpleBurn
{
  public class SBBurn 
    {

        //Converts An Item Identifier List To A File System Path
        [DllImport("Shell32.dll")]
        public static extern bool SHGetPathFromIDList(IntPtr ilPtr, string sPath); 

        //Retrieves A Pointer To The ITEMIDLIST Structure Of A Special Folder.
        [DllImport("shell32.dll", CharSet = CharSet.Auto)]
        private static extern uint SHGetSpecialFolderLocation(IntPtr hWnd, CSIDL nFolder, out IntPtr Pidl);
      
        private const int CSIDL_CDBURN_AREA = 59; //Burn Area API Constant
 
        private ICDBurn SBICDBurn; //ICDBurn Object

        private bool SBDisposedVal = false; //Detect Redundant Calls 
        private bool SBRecordableDrive; //Recording Drive Present

        private string SBriveLetter; //Recorder's Drive Letter
        private string SBStageFolder; //Folder Holding Files To Be Burnt

        /// <summary>
        /// Provide A Unique System - Independent Way To Identify Special Folders Used Frequently 
        /// By Applications, But Which May Not Have The Same Name Or Location On Any Given System. 
        /// </summary>
        public enum CSIDL
        {
            Desktop = 0x0000, // Desktop
            Internet = 0x0001, // Internet Explorer ( Icon On Desktop )
            Programs = 0x0002, // Start Menu\Programs
            Controls = 0x0003, // My Computer\Control Panel
            Printers = 0x0004, // My Computer\Printers
            Personal = 0x0005, // My Documents
            Favorites = 0x0006, // User Name\Favorites
            Startup = 0x0007, // Start Menu\Programs\Startup
            Recent = 0x0008, // User Name\Recent
            SendTo = 0x0009, // User Name\SendTo
            BitBucket = 0x000a, // Desktop\Recycle Bin
            StartMenu = 0x000b, // User Name\Start Menu
            MyDocuments = 0x000c, // Logical "My Documents" Desktop Icon
            MyMusic = 0x000d, // "My Music" Folder
            MyVideo = 0x000e, // "My Videos" Folder
            DesktopDirectory = 0x0010, // User Name\Desktop
            Drives = 0x0011, // My Computer
            Network = 0x0012, // Network Neighborhood (My Network Places)
            Nethood = 0x0013, // User Name\NetHood
            Fonts = 0x0014, // Windows\Fonts
            Templates = 0x0015,
            CommonStartMenu = 0x0016, // All Users\Start Menu
            CommonPrograms = 0x0017, // All Users\Start Menu\Programs
            CommonStartup = 0x0018, // All Users\Startup
            CommonDesktopDirectory = 0x0019, // All Users\Desktop
            AppData = 0x001a, // User Name\Application Data
            PrintHood = 0x001b, // User Name\PrintHood
            LocalAppData = 0x001c, // User Name\Local Settings\Applicaiton Data ( Non Roaming )
            AltStartup = 0x001d, // Non Localized Startup
            CommonAltStartup = 0x001e, // Non Localized Common Startup
            CommonFavorites = 0x001f,
            InternetCache = 0x0020,
            Cookies = 0x0021,
            History = 0x0022,
            CommonAppdata = 0x0023, // All Users\Application Data
            Windows = 0x0024, // GetWindowsDirectory()
            System = 0x0025, // GetSystemDirectory()
            ProgramFiles = 0x0026, // C:\Program Files
            MyPictures = 0x0027, // C:\Program Files\My Pictures
            Profile = 0x0028, // USERPROFILE
            SystemX86 = 0x0029, // X86 System Directory On RISC
            ProgramFilesX86 = 0x002a, // X86 C:\Program Files On RISC
            ProgramFilesCommon = 0x002b, // C:\Program Files\Common
            ProgramFilesCommonx86 = 0x002c, // X86 Program Files\Common On RISC
            CommonTemplates = 0x002d, // All Users\Templates
            CommonDocuments = 0x002e, // All Users\Documents
            CommonAdminTools = 0x002f, // All Users\Start Menu\Programs\Administrative Tools
            AdminTools = 0x0030, // User Name\Start Menu\Programs\Administrative Tools
            Connections = 0x0031, // Network And Dial-up Connections
            CommonMusic = 0x0035, // All Users\My Music
            CommonPictures = 0x0036, // All Users\My Pictures
            CommonVideo = 0x0037, // All Users\My Video
            Resources = 0x0038, // Resource Direcotry
            ResourcesLocalized = 0x0039, // Localized Resource Direcotry
            CommonOemLinks = 0x003a, // Links To All Users OEM specific apps
            CdBurnArea = 0x003b, // USERPROFILE\Local Settings\Application Data\Microsoft\CD Burning
            ComputersNearMe = 0x003d, // Computers Near Me ( Computered From Workgroup Membership )
            FlagCreate = 0x8000, // Combine With CSIDL_ Value To Force Folder Creation In SHGetFolderPath()
            FlagDontVerify = 0x4000, // Combine With CSIDL_ Value To Return An Unverified Folder Path
            FlagNoAlias = 0x1000, // Combine With CSIDL_ Value To Insure Non - Alias Versions Of The PIDL
            FlagPerUserInit = 0x0800, // Combine With CSIDL_ Value To Indicate Per - User Init ( eg. Upgrade )
            FlagMask = 0xFF00, // Mask For All Possible Flag Values
        }

        public enum ImapiErrorValues : uint //Some Of The Returned Errors By ICDBurn Interface
        {
            Ok = 0, //Everything Fine
            False = 1,
            PropertiesIgnored = 2147746304, //Ignored Unknown Property Passed
            BufferTooSmall = 2147746305, //Buffer Too Small
            NotOpened = 2147746315, //DiscMaster Has Not Been Opened Yet
            NotInitialized = 2147746316, //Recorder Object Not Initialized
            UserAbort = 2147746317, //Cancelled By The USer
            Generic = 2147746318, //Generic Error
            MediumNotPresent = 2147746319, //No Disk In Drive
            MediumInvalidType = 2147746320, //Disk Not Correct Type
            DeviceNoProperties = 2147746321, //Recorder Doesn't Support Properties
            DeviceNotAccessible = 2147746322, //Device Cannot Be Used / Already In Use
            DeviceNotPresent = 2147746323, //Device Not Present
            DeviceInvalidType = 2147746324, 
            InitializeWrite = 2147746325, //Drive Interface Cannot Be Initialized
            InitializeEndWrite = 2147746326, //Drive Interface Cannot Be Initilaized For Closing
            FileSystem = 2147746327, //File System Error
            FileAccess = 2147746328, //Error While Writing Image File
            DiskInfo = 2147746329, //Error While REading Disk Data
            TrackNotOpen = 2147746330, //Audio Track Not Open For Writing
            TrackOpen = 2147746331, //Audio Track Already Staged / Opened For Writing
            DiskFull = 2147746332, //Disk Is Full
            BadJolietName = 2147746333, //Badly named Element
            InvalidImage = 2147746334, //Image Is Corrupted / Dmaaged / Cleared
            NoActiveFormat = 2147746335, //Recordin Format Not Selected Yet
            NoActiveRecorder = 2147746336, //Recorder Not Selected Yet
            WrongFormat = 2147746337,
            AlreadyOpen = 2147746338, //DiscMaster Already Open
            WrongDisk = 2147746339, //Disk Was Removed From Active Recorder
            FileExists = 2147746340, //File Already Exists
            StashInUse = 2147746341, //Stash File Already In Use By Different Application
            DeviceStillInUse = 2147746342, //A Differnt Application IS Already Using The Device
            LossOfStreaming = 2147746343, //Content Streaming Was Lost
            CompressedStash = 2147746344, //Stash On Compressed Volume
            EncryptedStash = 2147746345, //Stash On Encrypted Volume
            NotEnoughDiskForStash = 2147746346, //Not Enough Free Space to Create Stash File
            RemovableStash = 2147746347, //Stash On Removable Volume
            CannotWriteToMedia = 2147746348, //Cannot Write To Media
            TrackNotBigEnough = 2147746349, //Track Not Big Enough
            BootImageAndNonBlankDisk = 2147746350 //Attempted To Create Boot Image On Non - Blank Disk
        }

      /// <summary>
      /// Set Upfront Properties
      /// </summary>
        public SBBurn()
        {
            if (!CreateInterface()) //Try To Create Interface
                throw new Exception("Cannot Create ICDBurn Interface"); //Cannot

            SBRecordableDrive = SBRecorderAvailable(); //Is Recorder Available 

            if (SBRecordableDrive) //If Available

                SBriveLetter = SBGetDriveLetter(); //Store Drive Letter Of Recordable Device

            SBStageFolder = GetBurnStagingAreaFolder(); //Set Stage Folder
        }

        /// <summary>
        /// Disposal Of Burner
        /// </summary>
        /// <param name="disposing"></param>
        protected void Dispose(bool disposing)
        {
            if (!this.SBDisposedVal)
            {
                if (disposing)
                {
                    if (SBICDBurn != null)
                    {
                        //Dispose Object, In Order To Be Able To Burn Again
                        Marshal.FinalReleaseComObject(SBICDBurn);

                        SBICDBurn = null;
                    }
                }
            }
            this.SBDisposedVal = true;
        }

        /// <summary> 
        /// Do We Have A Recordable Device 
        /// </summary> 
        public bool SBHasRecordableDrive
        {
            get 
            {
                return SBRecordableDrive; //Return Value - True Or False
            }
        }

        /// <summary> 
        /// Get Recorder's Drive Letter 
        /// </summary> 
        public string SBRecorderDrivePath
        {
            get 
            {
                return SBriveLetter; //Retrun Value
            }
        }

        /// <summary> 
        /// Get Temporary Burn Folder
        /// </summary> 
        public string SbStageAreaFolderPath
        {
            get 
            {
                return SBStageFolder; //Return Value
            }
        }
      

        /// <summary> 
        /// Copy From Stage To CD 
        /// </summary> 
        public ImapiErrorValues Burn(IntPtr h)
        {
            return (ImapiErrorValues)SBICDBurn.SBBurn(h); //Burn

        }


        /// <summary> 
        /// Create ICD_Burn Interface
        /// </summary> 
        private bool CreateInterface()
        {

            Guid SBShellCDBurnStr = new Guid("FBEB8A05-BEEE-4442-804e-409d6c4515e9"); //Shell Object 
            Guid SBInterfaceID_ICDBurn = new Guid("3d73a659-e5d0-4d42-afc0-5121ba425c8d"); //Interface IID

            Type ComType = Type.GetTypeFromCLSID(SBShellCDBurnStr); //Create COM Object Through It's GUID

            if (ComType == null) //If Empty
            {
                throw new Exception("Operating System Not Supported");
            }

            object SBICDBurnObj = null; //Object To "Hold" The Burn Object

            SBICDBurnObj = Activator.CreateInstance(ComType); //Create It

            SBICDBurn = (ICDBurn)SBICDBurnObj; //Set Value

            SBICDBurnObj = null; //Release

            if (SBICDBurn != null) 
            {
                return true; //Interface Exists
            }
            else
            {
                MessageBox.Show("Cannot Create Interface"); //Interface Doesn't Exist

                return false;
            }

        }

        /// <summary> 
        /// Method To Determine The SBRecordableDrivePresent Property
        /// And To Set The Result = To The Drive
        /// </summary> 
        private bool SBRecorderAvailable()
        {
            if (SBICDBurn == null) //If No Recorder
                return false; 

            int SBDrive = 0; //Initialize Drive
            int SBResult;

            SBResult = SBICDBurn.SBRecordableDrivePresent(ref SBDrive); //Get Drive

            return Convert.ToBoolean(SBDrive); //Convert To True / False
        }

        /// <summary>
        /// Get Drive Letter
        /// </summary>
        private string SBGetDriveLetter()
        {
            // Only one drive on the system can have "allow cd burning on this drive" set on it's 
            // properties. 

            if (SBHasRecordableDrive == false) //If No Drive Found
                return "No Available Drive";

            if (SBICDBurn == null) //If We Cannot Create ICDBurn Interface
                throw new Exception("ICDBurn is nothing!");

            string SBDrive = " "; //Initialize Drive Letter Variable

            uint SBResult = (uint)SBICDBurn.GetRecorderDriveLetter(SBDrive, 4); //Get Drive Letter

            if (SBResult != (uint)ImapiErrorValues.Ok) //If Everything Not OK
            {
                ImapiErrorValues SBErrorCode = (ImapiErrorValues)SBResult; //Get Returned Error Code
                    throw new Exception("Error Returned By IMAPI " + SBErrorCode);
            }

            //Make Returned Drive Letter String More Readable
            SBDrive = SBDrive.TrimEnd(new char[] { char.MinValue });

            return SBDrive; //Retrun Value
        }

        /// <summary>
        /// Get The Temporary burn Folder On The System
        /// </summary>
        public static string GetBurnStagingAreaFolder()
        {
            IntPtr pidl = Marshal.AllocHGlobal(1024); //Make Space

            SHGetSpecialFolderLocation(pidl, CSIDL.CdBurnArea, out pidl); //Get The Temporary Burn Folder

            string path = new string(char.MinValue, 260); //Create Enough Space In String

            bool result = SHGetPathFromIDList(pidl, path); //See If Possible

            if (result == false)
            {
                throw new Exception("No Staging Area Specified"); //No Staging Area Specified
            }
            Marshal.FreeHGlobal(pidl); //Free The Space
            
            path = path.TrimEnd(char.MinValue); //Format String Appropriately

            return path; //Return Value
      }
    }

  
}