using System;
using System.Drawing;
using System.Windows.Forms;

namespace SimpleBurn
{
    partial class frmSimpleBurn : System.Windows.Forms.Form
    {
        private SBBurn SBBurner; //Burner Object

        private string SBStage; //Stage

        private Point SbMouseOffset; //Mouse Starting Location

        public frmSimpleBurn()
        {
            // This call is required by the Windows Form Designer. 
            InitializeComponent();

            SBBurner = new SBBurn(); //Create SBBurn Object

            SBStage = SBBurner.SbStageAreaFolderPath; //Store CSIDL CdBurnArea Location

            if(SBStage == string.Empty) //If Not Found

                //Hard Code Default Location
                SBStage = "C:\\Documents and Settings\\Administrator\\Local Settings\\Application Data\\Microsoft\\CD Burning";

            this.lblHasBurner.Text = this.lblHasBurner.Text + SBBurner.SBHasRecordableDrive; //Recordable Device = True / False 

            this.lblDriveLetter.Text =  SBBurner.SBRecorderDrivePath; //Recorder Drive Letter

            this.txtStagingArea.Text = SBStage; //Display Stage Path
        }

        /// <summary>
        /// Add Complete Folders To Be Burnt To Stage
        /// </summary>
        private void AddFolder(string SBMainDir, System.IO.DirectoryInfo SBCurrDir, bool SBRecurse)
        {
            Application.DoEvents();
            this.lbFiles.Refresh();

            this.lbFiles.Items.Add(SBCurrDir.FullName); //Add Directory

            string SBStagePath; 

            if (SBMainDir == SBCurrDir.FullName)
            {
                SBStagePath = SBStage; //Stage Path
            }
            else
            {
                string SBRelPath = SBCurrDir.FullName.Substring(SBMainDir.Length + 1);

                SBStagePath = System.IO.Path.Combine(SBStage, SBRelPath);
               
                if (System.IO.Directory.Exists(SBStagePath) == false)
                {
                    System.IO.Directory.CreateDirectory(SBStagePath); //Create Directory
                }
            }

            foreach (System.IO.FileInfo file in SBCurrDir.GetFiles()) //Copy Files Found
            {
                System.IO.File.Copy(file.FullName, System.IO.Path.Combine(SBStagePath, file.Name));

                this.lbFiles.Items.Add(file.FullName); //Add To List
            }

            if (SBRecurse) //Loop Through Sub Directories
            {
                foreach (System.IO.DirectoryInfo dir in SBCurrDir.GetDirectories())
                {
                    AddFolder(SBMainDir, dir, true); //Add Folders
                }
            }

        }

        /// <summary>
        /// Delete All When Done
        /// </summary>
        /// <param name="directory"></param>
        private void DeleteFolder(System.IO.DirectoryInfo SBCurrDir)
        {
            foreach (System.IO.FileInfo file in SBCurrDir.GetFiles()) //Find Files On Stage
            {
                System.IO.File.Delete(file.FullName); //Delete
            }

            foreach (System.IO.DirectoryInfo folder in SBCurrDir.GetDirectories()) //Loop Through All Sub Dirs
            {
                DeleteFolder(folder); //Delete
            }
            if (SBCurrDir.FullName != SBStage)

                System.IO.Directory.Delete(SBCurrDir.FullName);
        }

        /// <summary>
        /// Remove All Folders When Done
        /// </summary>
        private void frmSimpleBurn_FormClosing(object sender, System.Windows.Forms.FormClosingEventArgs e)
        {
            if (SBBurner != null)
                DeleteFolder(new System.IO.DirectoryInfo(SBStage));

        }

        //Enable Form Dragging
        private void frmSimpleBurn_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left) //Left Button Pressed
            {
                Point SBMousePos = Control.MousePosition; //Get Location

                SBMousePos.Offset(SbMouseOffset.X, SbMouseOffset.Y); 

                Location = SBMousePos;
            }

        }

        /// <summary>
        /// Move Form, While Moving Mouse
        /// </summary>
        private void frmSimpleButton_MouseDown(object sender, MouseEventArgs e)
        {
            SbMouseOffset = new Point(-e.X, -e.Y); //Move Form

        }

        /// <summary>
        /// Create Background Skin
        /// </summary>
        private void frmSimpleBurn_Paint(object sender, PaintEventArgs e)
        {
            Bitmap bmpSBBack; //Create Bitmap Object

            bmpSBBack = (Bitmap)this.BackgroundImage; //Obtain Form Image

            bmpSBBack.MakeTransparent(Color.White); //Make Al White Pixels Transparent
        }

        /// <summary>
        /// Add Files
        /// </summary>
        private void picAddFiles_Click(object sender, EventArgs e)
        {
            OpenFileDialog SBOFD = new OpenFileDialog(); //New Open File Dialog

            string SBMyComputerGUID = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}"; //GUID Of My Computer

            SBOFD.InitialDirectory = SBMyComputerGUID; //Initial Directory

            SBOFD.Multiselect = true; //Can Select More Than One File

            System.Windows.Forms.DialogResult SBResult = SBOFD.ShowDialog(); 

            if (SBResult != System.Windows.Forms.DialogResult.OK) //If Not File Selected / Cancelled
                return; 

            foreach (string SBFile in SBOFD.FileNames) //Get All File Names
            {
                if (System.IO.File.Exists(SBFile))
                {
                    System.IO.File.Copy(SBFile, SBStage + "\\" + System.IO.Path.GetFileName(SBFile));  //Copy Selected Files To Stage Folder
                }
                this.lbFiles.Items.Add(SBFile); //Add To List
            }
        }

        /// <summary>
        /// Add Folders
        /// </summary>
        private void picAddFolders_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog SBOFD = new FolderBrowserDialog();

            SBOFD.RootFolder = Environment.SpecialFolder.MyComputer; //My Computer 

            SBOFD.ShowNewFolderButton = false; //Cannot Create New Folders

            System.Windows.Forms.DialogResult SBResult = SBOFD.ShowDialog();

            if (SBResult != System.Windows.Forms.DialogResult.OK) //If Not File Selected / Cancelled
                return;

            System.Windows.Forms.DialogResult SBResult2 = MessageBox.Show("Include Subdirectories", "Inlcude SubDirectories ", MessageBoxButtons.YesNoCancel); //Recursing 

            if (SBResult2 == DialogResult.Cancel) //No
                return;

            bool SBRecurse = (SBResult2 == DialogResult.Yes); //Yes

            AddFolder(SBOFD.SelectedPath, new System.IO.DirectoryInfo(SBOFD.SelectedPath), SBRecurse); //Loop
        }

        /// <summary>
        /// Start Burn Process
        /// </summary>
        private void picBurn_Click(object sender, EventArgs e)
        {
            SBBurner.Burn(this.Handle); //Burn!

            DeleteFolder(new System.IO.DirectoryInfo(SBStage)); //Delete Files On Stage Afterwards

            this.lbFiles.Items.Clear(); //Clear The ListBox
            this.lbFiles.Refresh();
        }

        private void picExit_Click(object sender, EventArgs e)
        {
            this.Close(); //Exit
        }

    }

}